@@ -1,5 +1,6 @@
 # -*- coding: utf-8 -*-
 
+import fnmatch
 import glob
 import locale
 import os
@@ -9,7 +10,32 @@ import subprocess as sp
 import sys
 import tempfile
 import time
-from os.path import join
+
+if sys.platform not in ("darwin", "win32"):
+	# Linux
+	import grp
+	import pwd
+
+try:
+	reloaded
+except NameError:
+	# First import. All fine
+	reloaded = 0
+else:
+	# Module is being reloaded. NOT recommended.
+	reloaded += 1
+	import warnings
+	warnings.warn("Module %s is being reloaded. This is NOT recommended." %
+				  __name__, RuntimeWarning)
+	warnings.warn("Implicitly reloading builtins", RuntimeWarning)
+	reload(__builtin__)
+	warnings.warn("Implicitly reloading os", RuntimeWarning)
+	reload(os)
+	warnings.warn("Implicitly reloading os.path", RuntimeWarning)
+	reload(os.path)
+	if sys.platform == "win32":
+		warnings.warn("Implicitly reloading win32api", RuntimeWarning)
+		reload(win32api)
 
 if sys.platform == "win32":
 	import ctypes
@@ -26,7 +52,7 @@ from encoding import get_encodings
 
 fs_enc = get_encodings()[1]
 
-os._listdir = os.listdir
+_listdir = os.listdir
 
 if sys.platform == "win32":
 	# Add support for long paths (> 260 chars)
@@ -35,7 +61,7 @@ if sys.platform == "win32":
 	import winerror
 	import win32api
 
-	__builtin__._open = __builtin__.open
+	_open = __builtin__.open
 
 
 	def retry_sharing_violation_factory(fn, delay=0.25, maxretries=20):
@@ -57,115 +83,116 @@ if sys.platform == "win32":
 
 
 	def open(path, *args, **kwargs):
-		return __builtin__._open(make_win32_compatible_long_path(path), *args,
+		""" Wrapper around __builtin__.open dealing with win32 long paths """
+		return _open(make_win32_compatible_long_path(path), *args,
 								 **kwargs)
 
 	__builtin__.open = open
 
 
-	os._access = os.access
+	_access = os.access
 
 	def access(path, mode):
-		return os._access(make_win32_compatible_long_path(path), mode)
+		return _access(make_win32_compatible_long_path(path), mode)
 
 	os.access = access
 
 
-	os.path._exists = os.path.exists
+	_exists = os.path.exists
 
 	def exists(path):
-		return os.path._exists(make_win32_compatible_long_path(path))
+		return _exists(make_win32_compatible_long_path(path))
 
 	os.path.exists = exists
 
 
-	os.path._isdir = os.path.isdir
+	_isdir = os.path.isdir
 
 	def isdir(path):
-		return os.path._isdir(make_win32_compatible_long_path(path))
+		return _isdir(make_win32_compatible_long_path(path))
 
 	os.path.isdir = isdir
 
 
-	os.path._isfile = os.path.isfile
+	_isfile = os.path.isfile
 
 	def isfile(path):
-		return os.path._isfile(make_win32_compatible_long_path(path))
+		return _isfile(make_win32_compatible_long_path(path))
 
 	os.path.isfile = isfile
 
 
 	def listdir(path):
-		return os._listdir(make_win32_compatible_long_path(path))
+		return _listdir(make_win32_compatible_long_path(path))
 
 
-	os._lstat = os.lstat
+	_lstat = os.lstat
 
 	def lstat(path):
-		return os._lstat(make_win32_compatible_long_path(path))
+		return _lstat(make_win32_compatible_long_path(path))
 
 	os.lstat = lstat
 
 
-	os._mkdir = os.mkdir
+	_mkdir = os.mkdir
 
 	def mkdir(path, mode=0777):
-		return os._mkdir(make_win32_compatible_long_path(path, 247), mode)
+		return _mkdir(make_win32_compatible_long_path(path, 247), mode)
 
 	os.mkdir = mkdir
 
 
-	os._makedirs = os.makedirs
+	_makedirs = os.makedirs
 
 	def makedirs(path, mode=0777):
-		return os._makedirs(make_win32_compatible_long_path(path, 247), mode)
+		return _makedirs(make_win32_compatible_long_path(path, 247), mode)
 
 	os.makedirs = makedirs
 
 
-	os._remove = os.remove
+	_remove = os.remove
 
 	def remove(path):
-		return os._remove(make_win32_compatible_long_path(path))
+		return _remove(make_win32_compatible_long_path(path))
 
 	os.remove = retry_sharing_violation_factory(remove)
 
 
-	os._rename = os.rename
+	_rename = os.rename
 
 	def rename(src, dst):
 		src, dst = [make_win32_compatible_long_path(path) for path in
 					(src, dst)]
-		return os._rename(src, dst)
+		return _rename(src, dst)
 
 	os.rename = retry_sharing_violation_factory(rename)
 
 
-	os._stat = os.stat
+	_stat = os.stat
 
 	def stat(path):
-		return os._stat(make_win32_compatible_long_path(path))
+		return _stat(make_win32_compatible_long_path(path))
 
 	os.stat = stat
 
 
-	os._unlink = os.unlink
+	_unlink = os.unlink
 
 	def unlink(path):
-		return os._unlink(make_win32_compatible_long_path(path))
+		return _unlink(make_win32_compatible_long_path(path))
 
 	os.unlink = retry_sharing_violation_factory(unlink)
 
 
-	win32api._GetShortPathName = win32api.GetShortPathName
+	_GetShortPathName = win32api.GetShortPathName
 
 	def GetShortPathName(path):
-		return win32api._GetShortPathName(make_win32_compatible_long_path(path))
+		return _GetShortPathName(make_win32_compatible_long_path(path))
 
 	win32api.GetShortPathName = GetShortPathName
 else:
 	def listdir(path):
-		paths = os._listdir(path)
+		paths = _listdir(path)
 		if isinstance(path, unicode):
 			# Undecodable filenames will still be string objects. Ignore them.
 			paths = filter(lambda path: isinstance(path, unicode), paths)
@@ -203,14 +230,11 @@ def expanduseru(path):
 		elif not 'HOMEPATH' in os.environ:
 			return path
 		else:
-			try:
-				drive = getenvu('HOMEDRIVE')
-			except KeyError:
-				drive = ''
-			userhome = join(drive, getenvu('HOMEPATH'))
+			drive = getenvu('HOMEDRIVE', '')
+			userhome = os.path.join(drive, getenvu('HOMEPATH'))
 
 		if i != 1: #~user
-			userhome = join(dirname(userhome), path[1:i])
+			userhome = os.path.join(dirname(userhome), path[1:i])
 
 		return userhome + path[i:]
 	return unicode(os.path.expanduser(path), fs_enc)
@@ -295,6 +319,11 @@ def expandvarsu(path):
 	return unicode(os.path.expandvars(path), fs_enc)
 
 
+def fname_ext(path):
+	""" Get filename and extension """
+	return os.path.splitext(os.path.basename(path))
+
+
 def get_program_file(name, foldername):
 	""" Get path to program file """
 	if sys.platform == "win32":
@@ -326,6 +355,23 @@ def getenvu(name, default = None):
 		return var if isinstance(var, unicode) else unicode(var, fs_enc)
 
 
+def getgroups(username=None, names_only=False):
+	"""
+	Return a list of groups that user is member of, or groups of current
+	process if username not given
+	
+	"""
+	if username is None:
+		groups = [grp.getgrgid(g) for g in os.getgroups()]
+	else:
+		groups = [g for g in grp.getgrall() if username in g.gr_mem]
+		gid = pwd.getpwnam(username).pw_gid
+		groups.append(grp.getgrgid(gid))
+	if names_only:
+		groups = [g.gr_name for g in groups]
+	return groups
+
+
 def islink(path):
 	"""
 	Cross-platform islink implementation.
@@ -342,10 +388,10 @@ def islink(path):
 def is_superuser():
 	if sys.platform == "win32":
 		if sys.getwindowsversion() >= (5, 1):
-			return ctypes.windll.shell32.IsUserAnAdmin()
+			return bool(ctypes.windll.shell32.IsUserAnAdmin())
 		else:
 			try:
-				return ctypes.windll.advpack.IsNTAdmin(0, 0)
+				return bool(ctypes.windll.advpack.IsNTAdmin(0, 0))
 			except Exception:
 				return False
 	else:
@@ -395,6 +441,19 @@ def make_win32_compatible_long_path(path, maxpath=259):
 	return path
 
 
+def mkstemp_bypath(path, dir=None, text=False):
+	"""
+	Wrapper around mkstemp that uses filename and extension from path as prefix 
+	and suffix for the temporary file, and the directory component as temporary
+	file directory if 'dir' is not given.
+	
+	"""
+	fname, ext = fname_ext(path)
+	if not dir:
+		dir = os.path.dirname(path)
+	return tempfile.mkstemp(ext, fname + "-", dir, text)
+
+
 def movefile(src, dst, overwrite=True):
 	""" Move a file to another location.
 	
@@ -616,8 +675,7 @@ def which(executable, paths = None):
 
 
 def whereis(filename):
-	for args in (["whereis", filename], ["locate", filename],
-				 ["ldconfig", "-p"]):
+	for args in (["whereis", filename], ["ldconfig", "-p"]):
 		try:
 			p = sp.Popen(args, stdout=sp.PIPE)
 			stdout, stderr = p.communicate()
@@ -627,12 +685,15 @@ def whereis(filename):
 			stdout_lines = stdout.strip().split(os.linesep)
 			if args[0] == "ldconfig":
 				for line in stdout_lines:
-					if filename in line:
-						return line.split("=>").pop().strip()
+					if fnmatch.fnmatch(line, filename):
+						# libxyz.so (libc6,x86_64) => /lib64/libxyz.so.1
+						return line.split("=>")[-1].strip()
 			else:
-				result = stdout_lines.pop().split(":", 1).pop().strip()
+				# $ whereis libxyz.so
+				# libxyz.so /lib64/libxyz.so /usr/lib/libxyz.so
+				result = stdout_lines[0].split(":", 1)[-1].split()
 				if result:
-					return result
+					return result[-1]
 
 
 if sys.platform == "win32" and sys.getwindowsversion() >= (6, ):
